@@ -1,15 +1,11 @@
 """ User interface for EdgePi ADC """
 
-
-from dataclasses import dataclass
 from enum import Enum
 from operator import attrgetter
 import logging
 import time
 
-
-from bitstring import pack
-from edgepi.adc.adc_query_lang import PropertyValue, ADCProperties
+from edgepi.adc.adc_query_lang import PropertyValue
 from edgepi.calibration.calibration_constants import CalibParam
 from edgepi.peripherals.spi import SpiDevice as SPI
 from edgepi.adc.adc_commands import ADCCommands
@@ -17,7 +13,6 @@ from edgepi.adc.adc_constants import (
     ADC1DataRate,
     ADC2DataRate,
     ADCChannel as CH,
-    ADCComs,
     ADCNum,
     ADCPower,
     ConvMode,
@@ -32,13 +27,22 @@ from edgepi.adc.adc_constants import (
     IDACMUX,
     IDACMAG,
     REFMUX,
+    ADC2REFMUX,
     RTDModes,
+    ADC1RtdConfig,
+    ADC2RtdConfig,
     AllowedChannels,
+    AnalogIn,
 )
-from edgepi.adc.adc_voltage import code_to_voltage, check_crc, code_to_temperature
+from edgepi.adc.adc_voltage import (
+    code_to_voltage,
+    code_to_temperature,
+    code_to_voltage_single_ended
+)
+from edgepi.utilities.crc_8_atm import check_crc
 from edgepi.gpio.edgepi_gpio import EdgePiGPIO
-from edgepi.gpio.gpio_configs import ADCPins
-from edgepi.utilities.utilities import filter_dict
+from edgepi.gpio.gpio_configs import ADCPins, RTDPins
+from edgepi.utilities.utilities import filter_dict, filter_dict_list_key_val
 from edgepi.reg_helper.reg_helper import OpCode, apply_opcodes
 from edgepi.adc.adc_multiplexers import (
     generate_mux_opcodes,
@@ -46,137 +50,20 @@ from edgepi.adc.adc_multiplexers import (
 )
 from edgepi.adc.adc_conv_time import expected_initial_time_delay, expected_continuous_time_delay
 from edgepi.adc.adc_status import get_adc_status
-from edgepi.calibration.edgepi_eeprom import EdgePiEEPROM
+from edgepi.eeprom.edgepi_eeprom import EdgePiEEPROM
+from edgepi.adc.adc_state import ADCState
+from edgepi.adc.adc_exceptions import (
+    ADCRegisterUpdateError,
+    VoltageReadError,
+    ContinuousModeError,
+    RTDEnabledError,
+    InvalidDifferentialPairError,
+    CalibKeyMissingError
+)
 
 _logger = logging.getLogger(__name__)
 
 
-@dataclass
-class ADCReadFields:
-    """
-    ADC state properties specific to each of ADC1 and ADC2
-    """
-
-    conversion_mode: PropertyValue
-    data_rate: PropertyValue
-    mux_p: PropertyValue
-    mux_n: PropertyValue
-
-
-class ADCState:
-    """ADC state intended for reading by users"""
-
-    def __init__(self, reg_map: dict):
-        self.__reg_map = reg_map
-        self.adc_1: ADCReadFields = ADCReadFields(
-            self.__get_state(ADCProperties.CONV_MODE),
-            self.__get_state(ADCProperties.DATA_RATE_1),
-            self.__get_state(ADCProperties.ADC1_MUXP),
-            self.__get_state(ADCProperties.ADC1_MUXN),
-        )
-        self.adc_2: ADCReadFields = ADCReadFields(
-            PropertyValue("continuous", ADCProperties.CONV_MODE),
-            self.__get_state(ADCProperties.DATA_RATE_2),
-            self.__get_state(ADCProperties.ADC2_MUXP),
-            self.__get_state(ADCProperties.ADC2_MUXN),
-        )
-        self.filter_mode: PropertyValue = self.__get_state(ADCProperties.FILTER_MODE)
-        self.status_byte: PropertyValue = self.__get_state(ADCProperties.STATUS_MODE)
-        self.checksum_mode: PropertyValue = self.__get_state(ADCProperties.CHECK_MODE)
-        self.rtd_on: bool = self.__get_rtd_state()
-
-    def __query_state(self, adc_property: ADCProperties) -> PropertyValue:
-        """
-        Read the current state of configurable ADC properties
-
-        Args:
-            `adc_property` (ADCProperties): ADC property whose state is to be read
-            `reg_map`: register map formatted as {addx (int): value (int)}
-
-        Returns:
-            `PropertyValue`: information about the current value of this property
-        """
-        # value of this adc_property's register
-        reg_value = self.__reg_map[adc_property.value.addx]
-
-        # get value of bits corresponding to this property by letting through only the bits
-        # that were "masked" when setting this property (clear all bits except the property bits)
-        adc_property_bits = (~adc_property.value.mask) & reg_value
-
-        # name of current value of this adc_property
-        adc_property_value = adc_property.value.values[adc_property_bits]
-        _logger.debug(
-            (
-                f"query_state: query_property='{adc_property}',"
-                " adc_property_bits={hex(adc_property_bits)},"
-                f" adc_property_value='{adc_property_value}'"
-            )
-        )
-
-        return adc_property_value
-
-    def __get_state(self, adc_property: ADCProperties) -> PropertyValue:
-        """
-        Read the current state of configurable ADC properties
-
-        Args:
-            `adc_property` (ADCProperties): ADC property whose state is to be read
-
-        Returns:
-            `PropertyValue`: information about the current value of this mode
-        """
-        return self.__query_state(adc_property)
-
-    def __get_current_rtd_state(self) -> dict[str, PropertyValue]:
-        return {
-            "adc_1_analog_in": self.adc_1.mux_p.code,
-            "adc_1_mux_n": self.adc_1.mux_n.code,
-            "idac_1_mux": self.__get_state(ADCProperties.IDAC1_MUX).code,
-            "idac_2_mux": self.__get_state(ADCProperties.IDAC2_MUX).code,
-            "idac_1_mag": self.__get_state(ADCProperties.IDAC1_MAG).code,
-            "idac_2_mag": self.__get_state(ADCProperties.IDAC2_MAG).code,
-            "pos_ref_inp": self.__get_state(ADCProperties.REFMUX_POS).code,
-            "neg_ref_inp": self.__get_state(ADCProperties.REFMUX_NEG).code,
-        }
-
-    def __get_rtd_state(self) -> bool:
-        """
-        Get on/off RTD state.
-        """
-        return self.__get_current_rtd_state() == RTDModes.RTD_ON.value
-
-
-class ADCStateMissingMap(Exception):
-    """ "Raised if ADCState.get_map() is called before ADCState has been assigned a cached state"""
-
-
-class ADCRegisterUpdateError(Exception):
-    """Raised when a register update fails to set register to expected value"""
-
-
-class VoltageReadError(Exception):
-    """Raised if a voltage read fails to return the expected number of bytes"""
-
-
-class ContinuousModeError(Exception):
-    """Raised when `read_voltage` is called and ADC is not in CONTINUOUS conversion mode"""
-
-
-class RTDEnabledError(Exception):
-    """Raised when user attempts to set ADC configuration that conflicts with RTD mode"""
-
-
-class InvalidDifferentialPairError(Exception):
-    """
-    Raised when calibration values are attempted to be retrieved for an invalid Differential
-    read pair
-    """
-
-
-class CalibKeyMissingError(Exception):
-    """Raised when calibration values are missing from EEPROM dictionary"""
-
-
 class EdgePiADC(SPI):
     """
     EdgePi ADC device
@@ -196,9 +83,18 @@ class EdgePiADC(SPI):
     RTD_SENSOR_RESISTANCE = 100 # RTD sensor resistance value (Ohms)
     RTD_SENSOR_RESISTANCE_VARIATION = 0.385 # RTD sensor resistance variation (Ohms/°C)
 
-    # TODO: this should be part of eeprom_data. Retrieve from eeprom_data in calling
-    # functions when available
-    r_ref = 1326.20
+    __analog_in_to_adc_in_map = {
+        AnalogIn.AIN1: CH.AIN0,
+        AnalogIn.AIN2: CH.AIN1,
+        AnalogIn.AIN3: CH.AIN2,
+        AnalogIn.AIN4: CH.AIN3,
+        AnalogIn.AIN5: CH.AIN4,
+        AnalogIn.AIN6: CH.AIN5,
+        AnalogIn.AIN7: CH.AIN6,
+        AnalogIn.AIN8: CH.AIN7,
+        AnalogIn.AINCOM : CH.AINCOM,
+        AnalogIn.FLOAT : CH.FLOAT
+    }
 
     def __init__(
         self,
@@ -222,8 +118,10 @@ class EdgePiADC(SPI):
 
         # Load eeprom data and generate dictionary of calibration dataclass
         eeprom = EdgePiEEPROM()
-        eeprom_data  = eeprom.get_edgepi_reserved_data()
-        self.adc_calib_params = eeprom_data.adc_calib_params
+        eeprom_data  = eeprom.read_edgepi_data()
+        self.adc_calib_params = {ADCNum.ADC_1:eeprom_data.adc1_calib_params.extract_ch_dict(),
+                                 ADCNum.ADC_2:eeprom_data.adc2_calib_params.extract_ch_dict(),}
+        self.r_ref = eeprom_data.rtd_calib_params.rtd_resistor
 
         self.adc_ops = ADCCommands()
         self.gpio = EdgePiGPIO()
@@ -233,7 +131,7 @@ class EdgePiADC(SPI):
         self.__config(checksum_mode=CheckMode.CHECK_BYTE_CRC)
         # TODO: adc reference should ba a config that customer passes depending on the range of
         # voltage they are measuring. To be changed later when range config is implemented
-        self.set_adc_reference(ADCReferenceSwitching.GND_SW1.value)
+        # self.set_adc_reference(ADCReferenceSwitching.GND_SW1.value)
 
         # user updated rtd hardware constants
         self.rtd_sensor_resistance = (
@@ -253,10 +151,10 @@ class EdgePiADC(SPI):
         Restore ADC to custom EdgePi configuration
         """
         # turn RTD off to allow updates in case RTD is on
-        self.rtd_mode(enable=False)
+        self.set_rtd(set_rtd=False)
         self.__config(
-            adc_1_analog_in=CH.FLOAT,
-            adc_2_analog_in=CH.FLOAT,
+            adc_1_ch=CH.FLOAT,
+            adc_2_ch=CH.FLOAT,
             adc_1_mux_n=CH.AINCOM,
             adc_2_mux_n=CH.AINCOM,
             checksum_mode=CheckMode.CHECK_BYTE_CRC,
@@ -313,12 +211,9 @@ class EdgePiADC(SPI):
         """
         if num_regs < 1:
             raise ValueError("Number of registers to read must be at least 1")
-
         code = self.adc_ops.read_register_command(start_addx.value, num_regs)
-        _logger.debug(f"ADC __read_register -> data in: {code}")
         out = self.transfer(code)
-        _logger.debug(f"ADC __read_register -> data out: {out}")
-
+        _logger.debug(f"__read_register: received {out}")
         # first 2 entries are null bytes
         return out[2:]
 
@@ -334,14 +229,24 @@ class EdgePiADC(SPI):
         """
         if len(data) < 1:
             raise ValueError("Number of registers to write to must be at least 1")
-
         code = self.adc_ops.write_register_command(start_addx.value, data)
-        _logger.debug(f"ADC __write_register -> data in: {code}")
+        _logger.debug(f"__write_register: sending {code}")
         out = self.transfer(code)
-        _logger.debug(f"ADC __write_register -> data out: {out}")
-
         return out
 
+    def __set_rtd_pin(self, enable: bool = False):
+        """
+        Set or clear RTD_EN pin
+        Args:
+            enable (bool): RTD enable flag
+        Return:
+            None
+        """
+        # pylint: disable=expression-not-assigned
+        self.gpio.set_pin_state(RTDPins.RTD_EN.value) if enable else \
+        self.gpio.clear_pin_state(RTDPins.RTD_EN.value)
+
+# TODO: To be deleted
     def set_adc_reference(self, reference_config: ADCReferenceSwitching = None):
         """
         Setting ADC referene terminal state. pin 18 and 23 labeled IN GND on the enclosure. It can
@@ -382,7 +287,6 @@ class EdgePiADC(SPI):
         pulse conversion mode, this method does not need to be called before
         performing reads.
         """
-
         # get state for configs relevant to conversion delay
         state = self.get_state()
         conv_mode = state.adc_1.conversion_mode.code
@@ -398,12 +302,11 @@ class EdgePiADC(SPI):
             (
                 f"\nComputed time delay = {conv_delay} (ms) with the following config opcodes:\n"
                 f"adc_num={adc_num}, conv_mode={hex(conv_mode.value.op_code)}, "
-                f"data_rate={hex(data_rate.value.op_code)}"
+                f"data_rate={hex(data_rate.value.op_code)}, "
                 f"filter_mode={hex(filter_mode.value.op_code)}\n"
             )
         )
         self.__send_start_command(adc_num)
-
         # apply delay for first conversion
         time.sleep(conv_delay / 1000)
 
@@ -430,20 +333,14 @@ class EdgePiADC(SPI):
                 (status_byte, voltage_data_bytes, check_byte)
         """
         read_data = self.__read_data(adc_num, ADC_VOLTAGE_READ_LEN)
-
         if len(read_data) - 1 != ADC_VOLTAGE_READ_LEN:
             raise VoltageReadError(
                 f"Voltage read failed: incorrect number of bytes ({len(read_data)}) retrieved"
             )
-
         status_code = read_data[1]
-
         voltage_code = read_data[2 : (2 + adc_num.value.num_data_bytes)]
-
         check_code = read_data[6]
-
         check_crc(voltage_code, check_code)
-
         return status_code, voltage_code, check_code
 
     @staticmethod
@@ -487,8 +384,7 @@ class EdgePiADC(SPI):
             raise ValueError("Cannot retrieve calibration values for channel in float mode")
 
         calib_key = mux_p.value if mux_n.code == CH.AINCOM else self.__get_diff_id(mux_p, mux_n)
-
-        calibs = adc_calibs.get(calib_key)
+        calibs = adc_calibs[calib_key]
 
         if calibs is None:
             _logger.error("Failed to find ADC calibration values")
@@ -502,9 +398,8 @@ class EdgePiADC(SPI):
 
         return calibs
 
-    def __continuous_time_delay(self, adc_num: ADCNum):
+    def __continuous_time_delay(self, adc_num: ADCNum, state: ADCState):
         """Compute and enforce continuous conversion time delay"""
-        state = self.get_state()
         # get continuous mode time delay and wait here (delay is needed between each conversion)
         data_rate = (
             state.adc_1.data_rate.code if adc_num == ADCNum.ADC_1 else state.adc_2.data_rate.code
@@ -513,9 +408,8 @@ class EdgePiADC(SPI):
 
         time.sleep(delay / 1000)
 
-    def __check_adc_1_conv_mode(self):
+    def __check_adc_1_conv_mode(self, state: ADCState):
         # assert adc is in continuous mode
-        state = self.get_state()
         if state.adc_1.conversion_mode.code != ConvMode.CONTINUOUS:
             raise ContinuousModeError(
                 "ADC1 must be in CONTINUOUS conversion mode in order to call this method."
@@ -533,21 +427,31 @@ class EdgePiADC(SPI):
         Returns:
             `float`: input voltage (V) read from the indicated ADC
         """
+        single_ended = False
+        state = self.get_state()
         if adc_num == ADCNum.ADC_1:
-            self.__check_adc_1_conv_mode()
+            self.__check_adc_1_conv_mode(state)
+            # Check whether the ADC is either in single-ended or differential
+            if state.adc_1.mux_n.code == CH.AINCOM:
+                single_ended = True
+        else:
+            if state.adc_2.mux_n.code == CH.AINCOM:
+                single_ended = True
 
-        self.__continuous_time_delay(adc_num)
+        self.__continuous_time_delay(adc_num, state)
 
         status_code, voltage_code, _ = self.__voltage_read(adc_num)
 
         # log STATUS byte
         status = get_adc_status(status_code)
-        _logger.debug(f"Logging STATUS byte:\n{status}")
-
-        calibs = self.__get_calibration_values(self.adc_calib_params, adc_num)
+        _logger.debug(f" read_voltage: Logging STATUS byte:\n{status}")
 
+        calibs = self.__get_calibration_values(self.adc_calib_params[adc_num], adc_num)
+        _logger.debug(f" read_voltage: gain {calibs.gain}, offset {calibs.offset}")
         # convert from code to voltage
-        return code_to_voltage(voltage_code, adc_num.value, calibs)
+
+        return code_to_voltage_single_ended(voltage_code, adc_num.value, calibs) if single_ended\
+            else code_to_voltage(voltage_code, adc_num.value, calibs)
 
 
     def read_rtd_temperature(self):
@@ -559,23 +463,24 @@ class EdgePiADC(SPI):
         Returns:
             `float`: RTD measured temperature (°C)
         """
-        self.__check_adc_1_conv_mode()
+        state = self.get_state()
+        adc_num = state.rtd_adc
+        if adc_num == ADCNum.ADC_1:
+            self.__check_adc_1_conv_mode(state)
+        self.__continuous_time_delay(adc_num, state)
 
-        self.__continuous_time_delay(ADCNum.ADC_1)
+        _, voltage_code, _ = self.__voltage_read(adc_num)
 
-        _, voltage_code, _ = self.__voltage_read(ADCNum.ADC_1)
-
-        # TODO: get RTD calibs from eeprom once added
         return code_to_temperature(
             voltage_code,
             self.r_ref,
             self.rtd_sensor_resistance,
-            self.rtd_sensor_resistance_variation
+            self.rtd_sensor_resistance_variation,
+            adc_num
         )
 
-    def __enforce_pulse_mode(self):
+    def __enforce_pulse_mode(self, state: ADCState):
         # assert adc is in PULSE mode (check ADCState) -> set to PULSE if not
-        state = self.get_state()
         if state.adc_1.conversion_mode.code != ConvMode.PULSE:
             self.__config(conversion_mode=ConvMode.PULSE)
 
@@ -589,7 +494,13 @@ class EdgePiADC(SPI):
         Returns:
             `float`: input voltage (V) read from ADC1
         """
-        self.__enforce_pulse_mode()
+        state = self.get_state()
+        single_ended = False
+        # Check whether the ADC is either in single-ended or differential
+        if state.adc_1.mux_n.code == CH.AINCOM:
+            single_ended = True
+
+        self.__enforce_pulse_mode(state)
 
         # send command to trigger conversion
         self.start_conversions(ADCNum.ADC_1)
@@ -599,12 +510,15 @@ class EdgePiADC(SPI):
 
         # log STATUS byte
         status = get_adc_status(status_code)
-        _logger.debug(f"Logging STATUS byte:\n{status}")
+        _logger.debug(f"single_sample: Logging STATUS byte:\n{status}")
 
-        calibs = self.__get_calibration_values(self.adc_calib_params, ADCNum.ADC_1)
+        calibs = self.__get_calibration_values(self.adc_calib_params[ADCNum.ADC_1], ADCNum.ADC_1)
 
         # convert from code to voltage
-        return code_to_voltage(voltage_code, ADCNum.ADC_1.value, calibs)
+        _logger.debug(f" read_voltage: code {voltage_code}")
+        _logger.debug(f" read_voltage: gain {calibs.gain}, offset {calibs.offset}")
+        return code_to_voltage_single_ended(voltage_code, ADCNum.ADC_1.value, calibs) if \
+            single_ended  else code_to_voltage(voltage_code, ADCNum.ADC_1.value, calibs)
 
     def single_sample_rtd(self):
         """
@@ -615,20 +529,23 @@ class EdgePiADC(SPI):
         Returns:
             `float`: RTD measured temperature (°C)
         """
-        self.__enforce_pulse_mode()
+        state = self.get_state()
+        adc_num = state.rtd_adc
+        if adc_num == ADCNum.ADC_1:
+            self.__enforce_pulse_mode(state)
 
         # send command to trigger conversion
-        self.start_conversions(ADCNum.ADC_1)
+        self.start_conversions(adc_num)
 
         # send command to read conversion data.
-        _, voltage_code, _ = self.__voltage_read(ADCNum.ADC_1)
+        _, voltage_code, _ = self.__voltage_read(adc_num)
 
-        # TODO: get RTD calibs from eeprom once added
         return code_to_temperature(
             voltage_code,
             self.r_ref,
             self.rtd_sensor_resistance,
-            self.rtd_sensor_resistance_variation
+            self.rtd_sensor_resistance_variation,
+            adc_num
         )
 
     def reset(self):
@@ -640,12 +557,16 @@ class EdgePiADC(SPI):
         self.transfer(self.adc_ops.reset_adc())
         self.__reapply_config()
 
-    def __is_data_ready(self):
+    def __is_data_ready(self, adc_num: ADCNum):
         # pylint: disable=unused-private-member
         # required for integration testing in test_conversion_times.py
         """Utility for testing conversion times, returns True if ADC indicates new voltage data"""
-        read_data = self.transfer([ADCComs.COM_RDATA1.value] + [255] * 6)
-        return read_data[1] & 0b01000000
+        read_data = self.transfer([adc_num.value.read_cmd] + [255] * 6)
+        if adc_num is ADCNum.ADC_1:
+            ready = (read_data[1] & 0b01000000) == 0b01000000
+        if adc_num is ADCNum.ADC_2:
+            ready = (read_data[1] & 0b10000000) == 0b10000000
+        return ready
 
     def __read_registers_to_map(self):
         """
@@ -660,26 +581,34 @@ class EdgePiADC(SPI):
 
         # build dict with (register addx : register_value) pairs.
         reg_dict = {addx + i: reg_values[i] for i in range(ADC_NUM_REGS)}
-        _logger.debug(f"__read_registers_to_map: {reg_dict}")
+        # _logger.debug(f"__read_registers_to_map: {reg_dict}")
 
         return reg_dict
 
     def __is_rtd_on(self):
         """
-        Get state of RTD_EN pin (on/off), to use for deciding which
-        ADC channels are available for reading.
-
-        Returns:
-            `bool`: True if RTD_EN pin is on, False otherwise
+        Check whether the RTD is enabled or disabled
+        Args:
+            N/A
+        Return:
+            rtd_pin_state (bool): True RTD enabled, False RTD disabled
         """
-        # TODO: this should use self.get_state() instead
-        _logger.debug("Checking RTD status")
-        idac_reg = self.__get_register_map()
-        idac_mag = pack("uint:8", idac_reg.get(ADCReg.REG_IDACMAG.value))
-        idac_1 = idac_mag[4:].uint
-        status = idac_1 != 0x0
-        _logger.debug(f"RTD enabled: {status}")
-        return status
+        pin_state = self.gpio.read_pin_state(RTDPins.RTD_EN.value)
+        pin_dir = self.gpio.get_pin_direction(RTDPins.RTD_EN.value)
+        if pin_state and not pin_dir:
+            return True
+        return False
+
+    def __get_rtd_state(self):
+        """
+        Get RTD state this includes the corrent mode (on/off) and the adc type being used
+        (adc1 or adc2).
+        Returns:
+            rtd_mode (dictionary): RTD mode dictionary {"name of config": op_codes}
+            rtd_adc (ADCNum): None, no adc attached to RTD, ADCNum.ADC1 or ADCNum.ADC1
+        """
+        state = self.get_state()
+        return state.rtd_mode, state.rtd_adc
 
     # by default set mux_n's to AINCOM. For diff_mode and rtd_mode, pass in custom mapping.
     # Case 1: user sets to adc_x read ch_x -> mux_n auto mapped to AINCOM
@@ -717,9 +646,7 @@ class EdgePiADC(SPI):
             adc_2_mux_n = None
 
         # no multiplexer config to update
-        # TODO: refactor filter_dict to take list arg
-        args = filter_dict(locals(), "self", None)
-        args = filter_dict(args, "override_rtd_validation")
+        args = filter_dict_list_key_val(locals(), ["self", "override_rtd_validation"], [None])
         if not args:
             return []
 
@@ -754,18 +681,18 @@ class EdgePiADC(SPI):
         mux_n = diff_mode.value.mux_n
         mux_properties = {
             ADCNum.ADC_1: {
-                "adc_1_analog_in": mux_p,
+                "adc_1_ch": mux_p,
                 "adc_1_mux_n": mux_n,
             },
             ADCNum.ADC_2: {
-                "adc_2_analog_in": mux_p,
+                "adc_2_ch": mux_p,
                 "adc_2_mux_n": mux_n,
             },
         }
         diff_update = mux_properties[adc]
         self.__config(**diff_update)
 
-    def __validate_no_rtd_conflict(self, updates: dict):
+    def  __validate_no_rtd_conflict(self, updates: dict):
         """
         Checks no RTD related properties are being updated if RTD mode is enabled
 
@@ -775,42 +702,86 @@ class EdgePiADC(SPI):
         Raises:
             `RTDEnabledError`: if RTD is enabled and an RTD related property is in updates
         """
-        # ADC2 channel setting conflicts with RTD handled during channel mapping
-        if not self.__is_rtd_on():
-            return
+        rtd_mode, rtd_adc = self.__get_rtd_state()
+        if rtd_mode is None or rtd_mode == RTDModes.RTD_OFF:
+            return None
 
-        rtd_properties = RTDModes.RTD_ON.value.keys()
+        # rtd_properties RTDModes.RTD1_ON or RTDModes.RTD2_ON
+        rtd_properties = rtd_mode.value | (ADC1RtdConfig.ON.value if rtd_adc == ADCNum.ADC_1 else\
+                                           ADC2RtdConfig.ON.value)
         for update in updates:
             if update in rtd_properties:
                 raise RTDEnabledError(
                     f"ADC property '{update}' cannot be updated while RTD is enabled"
                 )
+        return None
 
-    def rtd_mode(self, enable: bool):
+    def __check_adc_pins(self):
         """
-        Enable or disable RTD. Note, this will reconfigure ADC2 to read FLOAT input channel
-        if ADC2 is configured to read RTD pins AIN4-AIN7. To read RTD values aftering enabling
-        RTD mode here, call either `read_voltage` or `single_sample`, depending on which
-        conversion mode ADC1 is currently configured to.
+        check pin mux of each adc
+        Args:
+            N/A
+        Return:
+            channels
+        """
+        state = self.get_state()
+        adc_1_muxs = [state.adc_1.mux_n.code.value, state.adc_1.mux_p.code.value]
+        adc_2_muxs = [state.adc_2.mux_n.code.value, state.adc_2.mux_p.code.value]
+        return adc_1_muxs, adc_2_muxs
+
+    def __get_rtd_on_update_config(self, muxs: list, adc_num: ADCNum):
+        """
+        generate a dictionary of config parameters to enable RTD using specified ADC
+        Args:
+            muxs (list): list of input multiplexer of other ADC [negative input, positive input]
+            adc_num (ADCNum): ADC to read RTD
+        Return:
+            updates (dictionary): configuration dictionary for enabling RTD
+                                    {name_of_config : op_codes}
+        """
+        if any(mux not in [x.value for x in AllowedChannels.RTD_ON.value] for mux in muxs):
+            updates = RTDModes.RTD_ON.value |\
+            (ADC2RtdConfig.OFF.value if adc_num == ADCNum.ADC_1 else ADC1RtdConfig.OFF.value) |\
+            (ADC1RtdConfig.ON.value if adc_num ==ADCNum.ADC_1 else ADC2RtdConfig.ON.value)
+        else:
+            updates = RTDModes.RTD_ON.value |\
+            (ADC1RtdConfig.ON.value if adc_num ==ADCNum.ADC_1 else ADC2RtdConfig.ON.value)
+        return updates
+
+
+    def __get_rtd_off_update_config(self, adc_num: ADCNum):
+        """
+        generate update config dictionary to send to the config method
+        Args:
+            adc_num (ADCNum): ADC number
+        Return:
+            updates (dictionary): configuration dictionary for disabling RTD
+        """
+        updates = RTDModes.RTD_OFF.value |\
+        (ADC1RtdConfig.OFF.value if adc_num == ADCNum.ADC_1  else ADC2RtdConfig.OFF.value)
+        return updates
+
+    def set_rtd(self, set_rtd: bool, adc_num: ADCNum = ADCNum.ADC_2):
+        """
+        Enable/Disable RTD with ADC type passed as arguments.
+
+        NOTE: This function enforces only one ADC to be attached to RTD circuit
 
         Args:
-            `enable` (bool): True to enable RTD, False to disable
+            `set_rtd` (bool): True to enable RTD, False to disable
         """
-        # TODO: enable RTD_EN switching pin
-        if enable:
-            # check if adc_2 is reading RTD pins, remap channels if needed
-            mux_reg = self.__get_register_map()
-            adc2_mux = pack("uint:8", mux_reg.get(ADCReg.REG_ADC2MUX.value))
-            muxs = [adc2_mux[:4].uint, adc2_mux[4:].uint]
-            if any(mux not in [x.value for x in AllowedChannels.RTD_ON.value] for mux in muxs):
-                updates = RTDModes.RTD_ON.value | {
-                    "adc_2_analog_in": CH.FLOAT,
-                    "adc_2_mux_n": CH.AINCOM,
-                }
-            else:
-                updates = RTDModes.RTD_ON.value
+        if set_rtd:
+            # check inputs of both ADCs. Input 5~8 are only available for RTD reading
+            mux_1, mux_2 = self.__check_adc_pins()
+            # If inputs on another ADC are one of RTD only inputs, reconfigure the pins them to the
+            # default setting. And get RTD_ON configuration values
+            updates = self.__get_rtd_on_update_config(
+                           mux_2 if adc_num == ADCNum.ADC_1 else mux_1, adc_num)
+            # enable RTD pin to re-route internal circuit
+            self.__set_rtd_pin(set_rtd)
         else:
-            updates = RTDModes.RTD_OFF.value
+            updates = self.__get_rtd_off_update_config(adc_num)
+            self.__set_rtd_pin(set_rtd)
 
         self.__config(**updates, override_rtd_validation=True)
 
@@ -826,8 +797,8 @@ class EdgePiADC(SPI):
             `dict`: input multiplexer args formatted as {'mux_name': value}
         """
         mux_arg_names = {
-            "adc_1_analog_in": "adc_1_mux_p",
-            "adc_2_analog_in": "adc_2_mux_p",
+            "adc_1_ch": "adc_1_mux_p",
+            "adc_2_ch": "adc_2_mux_p",
             "adc_1_mux_n": "adc_1_mux_n",
             "adc_2_mux_n": "adc_2_mux_n",
         }
@@ -839,8 +810,8 @@ class EdgePiADC(SPI):
 
     def __config(
         self,
-        adc_1_analog_in: CH = None,
-        adc_2_analog_in: CH = None,
+        adc_1_ch: CH = None,
+        adc_2_ch: CH = None,
         adc_1_mux_n: CH = None,
         adc_2_mux_n: CH = None,
         adc_1_data_rate: ADC1DataRate = None,
@@ -855,6 +826,7 @@ class EdgePiADC(SPI):
         idac_2_mag: IDACMAG = None,
         pos_ref_inp: REFMUX = None,
         neg_ref_inp: REFMUX = None,
+        adc2_ref_inp: ADC2REFMUX = None,
         override_updates_validation: bool = False,
         override_rtd_validation: bool = False,
     ):
@@ -864,8 +836,8 @@ class EdgePiADC(SPI):
         `set_config()` for modifying settings instead.
 
         Args:
-            `adc_1_analog_in` (ADCChannel): input voltage channel to map to ADC1 mux_p
-            `adc_2_analog_in` (ADCChannel): input voltage channel to map to ADC2 mux_p
+            `adc_1_ch` (ADCChannel): input voltage channel to map to ADC1 mux_p
+            `adc_2_ch` (ADCChannel): input voltage channel to map to ADC2 mux_p
             `adc_1_mux_n` (ADCChannel): input voltage channel to map to ADC1 mux_n
             `adc_2_mux_n` (ADCChannel): input voltage channel to map to ADC1 mux_n
             `adc_1_data_rate` (ADC1DataRate): ADC1 data rate in samples per second
@@ -887,10 +859,9 @@ class EdgePiADC(SPI):
             `override_rtd_validation` (bool): turn off RTD property validation for RTD mode updates
         """
         # pylint: disable=unused-argument
-
         # filter out self and None args
         args = filter_dict(locals(), "self", None)
-        _logger.debug(f"__config: args after filtering out None defaults:\n\n{args}\n\n")
+        _logger.debug(f"__config: args after filtering out None defaults:\n{args}")
 
         # permit updates by rtd_mode() to turn RTD off when it's on, validate other updates
         if not override_rtd_validation:
@@ -912,12 +883,11 @@ class EdgePiADC(SPI):
 
         # get current register values
         reg_values = self.__get_register_map()
-        _logger.debug(f"__config: register values before updates:\n\n{reg_values}\n\n")
+        _logger.debug(f"__config: register values before updates:\n{reg_values}")
 
         # get codes to update register values
-        _logger.debug(f"opcodes = {ops_list}")
         updated_reg_values = apply_opcodes(dict(reg_values), ops_list)
-        _logger.debug(f"__config: register values after updates:\n\n{reg_values}\n\n")
+        _logger.debug(f"__config: register values after updates:\n{reg_values}")
 
         # write updated reg values to ADC using a single write.
         data = [entry["value"] for entry in updated_reg_values.values()]
@@ -957,9 +927,9 @@ class EdgePiADC(SPI):
 
     def set_config(
         self,
-        adc_1_analog_in: CH = None,
+        adc_1_analog_in: AnalogIn = None,
         adc_1_data_rate: ADC1DataRate = None,
-        adc_2_analog_in: CH = None,
+        adc_2_analog_in: AnalogIn = None,
         adc_2_data_rate: ADC2DataRate = None,
         filter_mode: FilterMode = None,
         conversion_mode: ConvMode = None,
@@ -969,9 +939,9 @@ class EdgePiADC(SPI):
         Configure user accessible ADC settings, either collectively or individually.
 
         Args:
-            `adc_1_analog_in` (ADCChannel): the input voltage channel to measure via ADC1
+            `adc_1_analog_in` (AnalogIn): the input voltage channel to measure via ADC1
             `adc_1_data_rate` (ADC1DataRate): ADC1 data rate in samples per second
-            `adc_2_analog_in` (ADCChannel): the input voltage channel to measure via ADC2
+            `adc_2_analog_in` (AnalogIn): the input voltage channel to measure via ADC2
             `adc_2_data_rate` (ADC2DataRate): ADC2 data rate in samples per second
             `filter_mode` (FilterMode): filter mode for ADC1.
                 Note this affects data rate. Please refer to module documentation
@@ -979,9 +949,19 @@ class EdgePiADC(SPI):
             `conversion_mode` (ConvMode): set conversion mode for ADC1.
             `override_updates_validation` (bool): set to True to skip update validation
         """
+        # pylint: disable=possibly-unused-variable
         # pylint: disable=unused-argument
+        adc_1_ch  = self.__analog_in_to_adc_in_map.get(adc_1_analog_in)
+        adc_2_ch  = self.__analog_in_to_adc_in_map.get(adc_2_analog_in)
 
-        args = filter_dict(locals(), "self", None)
+        if adc_1_ch is None and adc_1_analog_in is not None:
+            raise TypeError(f"set_config: wrong type passed for adc_1_analog_in: {adc_1_analog_in}")
+        if adc_2_ch is None and adc_2_analog_in is not None:
+            raise TypeError(f"set_config: wrong type passed for adc_2_analog_in: {adc_2_analog_in}")
+
+        args = filter_dict_list_key_val(locals(),
+                                        ["self", "adc_1_analog_in", "adc_2_analog_in"],
+                                        [None])
         self.__config(**args)
 
     def get_state(self, override_cache: bool = False) -> ADCState:
