@@ -5,7 +5,7 @@ Provides a class for interacting with the GPIO pins through GPIO peripheral
 import logging
 from edgepi.peripherals.gpio import GpioDevice
 from edgepi.gpio.gpio_constants import GpioDevPaths
-from edgepi.gpio.gpio_configs import DOUTPins, DINPins, generate_gpiochip_pin_info
+from edgepi.gpio.gpio_configs import DINPins, generate_gpiochip_pin_info
 
 _logger = logging.getLogger(__name__)
 
@@ -21,10 +21,9 @@ class EdgePiGPIOChip(GpioDevice):
                        DINPins.DIN4.value : 9,
                        DINPins.DIN5.value : 22,
                        DINPins.DIN6.value : 27,
-                       DINPins.DIN7.value : 3,
-                       DINPins.DIN8.value : 2,
-                       DOUTPins.DOUT1.value : 13,
-                       DOUTPins.DOUT2.value : 12}
+                       DINPins.DIN7.value : 10,
+                       DINPins.DIN8.value : 7
+                    }
 
     def __init__(self):
         super().__init__(GpioDevPaths.GPIO_CIHP_DEV_PATH.value)
